@@ -11,7 +11,16 @@ from cryspy import AtomSiteL, Crystal, Cell, SpaceGroup, \
     Pd2dMeas, Pd2dProc, Pd2d, value_error_to_string, \
     Setup, PhaseL, Range, DiffrnRadiation, Pd2dBackground, \
     Pd2dInstrResolution, DiffrnOrientMatrix, DiffrnReflnL, Phase, \
-    AtomElectronConfigurationL, MEM, PdPeakL, Pd2dPeakL, AtomSiteScatL
+    AtomElectronConfigurationL, MEM, PdPeakL, Pd2dPeakL, AtomSiteScatL, \
+    HessianMatrix
+
+from cryspy.A_functions_base.function_2_crystallography_base import \
+    calc_atoms_in_unit_cell
+from cryspy.A_functions_base.function_2_mem import \
+    calc_index_atom_symmetry_closest_to_fract_xyz
+
+from cryspy.D_functions_item_loop.function_1_section_from_density_point \
+    import calc_section_from_density_point
 
 import matplotlib.pyplot as plt
 
@@ -58,6 +67,8 @@ def w_for_presentation(obj, thread: QtCore.QThread) -> tuple:
         w_actions = action_diffrn_refln_l(obj, thread)
     elif isinstance(obj, AtomSiteL):
         w_actions = action_atom_site_l(obj, thread)
+    elif isinstance(obj, HessianMatrix):
+        w_actions = action_hessian_matrix(obj, thread)
     return docks, w_actions
 
 
@@ -73,6 +84,18 @@ def action_atom_site_l(obj: AtomSiteL, thread: QtCore.QThread):
     return w_actions
 
 
+def action_hessian_matrix(obj: HessianMatrix, thread: QtCore.QThread):
+    """Give actions for AtomSiteL."""
+    w_actions = []
+    if obj.is_defined():
+        qtb_1 = QtWidgets.QToolButton()
+        qtb_1.setText("Show correlation")
+        qtb_1.clicked.connect(lambda: run_function(
+            obj.report, (), thread))
+        w_actions.append(qtb_1)
+    return w_actions
+
+
 def action_atom_site_scat_l(obj: AtomSiteScatL, thread: QtCore.QThread):
     """Plot form factor."""
     w_actions = []
@@ -139,9 +162,13 @@ def action_diffrn_orient_matrix(obj: DiffrnOrientMatrix,
 
         def func_temp(obj):
             s_out = f"""Orientation matrix U:
- {obj.u_11: 8.5f} {obj.u_12: 8.5f} {obj.u_13: 8.5f}
- {obj.u_21: 8.5f} {obj.u_22: 8.5f} {obj.u_23: 8.5f}
- {obj.u_31: 8.5f} {obj.u_32: 8.5f} {obj.u_33: 8.5f}"""
+          a* [c,  a*]        c
+ X: {obj.u_11: 8.5f} {obj.u_12: 8.5f} {obj.u_13: 8.5f}
+ Y: {obj.u_21: 8.5f} {obj.u_22: 8.5f} {obj.u_23: 8.5f}
+ Z: {obj.u_31: 8.5f} {obj.u_32: 8.5f} {obj.u_33: 8.5f}
+
+axis 'X' is along incident beam;
+axis 'Z' is vertical direction."""
             return s_out
 
         qtb_1.clicked.connect(lambda: run_function(
@@ -267,6 +294,12 @@ def action_rhochi(obj: RhoChi, thread: QtCore.QThread):
             obj.refine, (False, "BFGS",), thread))
         w_actions.append(qtb_1)
 
+        qtb_1 = QtWidgets.QToolButton()
+        qtb_1.setText("Estimate Hessian")
+        qtb_1.clicked.connect(lambda: run_function(
+            obj.estimate_hessian, (), thread))
+        w_actions.append(qtb_1)
+
     elif not(flag_crystals & flag_experiments):
         if not flag_crystals:
             qtb_1 = QtWidgets.QToolButton()
@@ -304,6 +337,9 @@ def action_rhochi(obj: RhoChi, thread: QtCore.QThread):
         qtb_1.clicked.connect(lambda: run_function(
             obj.estimate_f_mag_for_diffrn, (), thread))
         w_actions.append(qtb_1)
+    if obj.is_attribute("hessian_matrix"):
+        w_actions_t = action_hessian_matrix(obj.hessian_matrix, thread)
+        w_actions.extend(w_actions_t)
     return w_actions
 
 
@@ -380,6 +416,121 @@ def action_mem(obj: MEM, thread: QtCore.QThread):
                                                    thread))
         w_actions.append(qtb_1)
 
+        qtb_1 = QtWidgets.QToolButton()
+        qtb_1.setText("Plot sections")
+
+        def func_plot(obj):
+            crystal = obj.crystals()[0]
+            space_group = crystal.space_group
+            f_s_g_s = space_group.full_space_group_symop
+            r_11 = numpy.array(f_s_g_s.r_11, dtype=float)
+            r_12 = numpy.array(f_s_g_s.r_12, dtype=float)
+            r_13 = numpy.array(f_s_g_s.r_13, dtype=float)
+            r_21 = numpy.array(f_s_g_s.r_21, dtype=float)
+            r_22 = numpy.array(f_s_g_s.r_22, dtype=float)
+            r_23 = numpy.array(f_s_g_s.r_23, dtype=float)
+            r_31 = numpy.array(f_s_g_s.r_31, dtype=float)
+            r_32 = numpy.array(f_s_g_s.r_32, dtype=float)
+            r_33 = numpy.array(f_s_g_s.r_33, dtype=float)
+
+            r_ij = (r_11, r_12, r_13, r_21, r_22, r_23, r_31, r_32, r_33)
+
+            b_1 = numpy.array(f_s_g_s.b_1, dtype=float)
+            b_2 = numpy.array(f_s_g_s.b_2, dtype=float)
+            b_3 = numpy.array(f_s_g_s.b_3, dtype=float)
+
+            b_i = (b_1, b_2, b_3)
+
+            atom_site = crystal.atom_site
+            fract_x = atom_site.numpy_fract_x
+            fract_y = atom_site.numpy_fract_y
+            fract_z = atom_site.numpy_fract_z
+            fract_xyz = (fract_x, fract_y, fract_z)
+
+            atom_label = atom_site.numpy_label
+
+            fract_uc_x, fract_uc_y, fract_uc_z, label_uc = \
+                calc_atoms_in_unit_cell(r_ij, b_i, fract_xyz, atom_label)
+
+            cell = crystal.cell
+            atom_site_susceptibility = crystal.atom_site_susceptibility
+
+            section = obj.section[0]
+            density_point = obj.density_point
+            mem_parameters = obj.mem_parameters
+
+            atom_x, atom_y, atom_label = section.calc_atoms(
+                cell, atom_site, f_s_g_s, distance_min=0.3)
+
+            den_chi_section, den_b_section = calc_section_from_density_point(
+                section, density_point, mem_parameters, cell, f_s_g_s,
+                atom_site, atom_site_susceptibility)
+
+            fract_atom_xyz = numpy.array(fract_xyz, dtype=float).transpose()
+
+            fract_sec_xyz = section.calc_fractions(cell, atom_site)
+            fract_sec_xyz = numpy.transpose(numpy.array(fract_sec_xyz,
+                                                        dtype=float))
+
+            n_atom_index, n_symmetry, distance = \
+                calc_index_atom_symmetry_closest_to_fract_xyz(
+                    fract_sec_xyz, fract_atom_xyz, r_ij, b_i, cell)
+            n_at_2d = numpy.transpose(n_atom_index.reshape(
+                section.points_x, section.points_y))
+
+            fig, (ax1, ax2) = plt.subplots(ncols=2, figsize=(4.2, 4.2),
+                                           dpi=300)
+            plt.set_cmap('Accent')
+            ax1.imshow(n_at_2d, interpolation='bilinear',
+                       extent=(-0.5*section.size_x, 0.5*section.size_x,
+                               -0.5*section.size_y, 0.5*section.size_y),
+                       alpha=0.1, origin="lower")
+
+            den_x = numpy.linspace(-0.5*section.size_x, 0.5*section.size_x,
+                                   section.points_x)
+            den_y = numpy.linspace(-0.5*section.size_y, 0.5*section.size_y,
+                                   section.points_y)
+
+            blk = '#000000'
+            ax1.contour(den_x, den_y, den_chi_section.transpose(),
+                        levels=[0.1, 0.5, 1., 5., 10., 50.],
+                        colors=[blk, blk, blk, blk, blk, blk], linewidths=0.5)
+
+            ax1.plot(atom_x, atom_y, 'ko', ms=3)
+            for _1, _2, _3 in zip(atom_x, atom_y, atom_label):
+                ax1.text(_1, _2, _3)
+            ax1.set_title(
+                f"Tensor. Max is {den_chi_section.max():.1f}")
+
+            # plt.set_cmap('RdBu')
+            ax2.imshow(n_at_2d, interpolation='bilinear',
+                       extent=(-0.5*section.size_x, 0.5*section.size_x,
+                               -0.5*section.size_y, 0.5*section.size_y),
+                       alpha=0.1, origin="lower")
+
+            hh = numpy.abs(den_b_section).max()
+            rd = '#FF0000'
+            ax2.contour(den_x, den_y, den_b_section.transpose(),
+                        levels=[-50., -10., -5., -1., -0.5, -0.1, 
+                                0.1, 0.5, 1., 5., 10., 50.],
+                        colors=[rd, rd, rd, rd, rd, rd,
+                                blk, blk, blk, blk, blk, blk], linewidths=0.5)
+            # ax2.imshow(den_b_section, interpolation='bilinear',
+            #            extent=(-0.5*section.size_x, 0.5*section.size_x,
+            #                    -0.5*section.size_y, 0.5*section.size_y),
+            #            vmin=-hh, vmax=hh,
+            #            alpha=1., origin="lower")
+            ax2.set_title(f"2channel. Max is {hh:.1f}")
+            ax2.plot(atom_x, atom_y, 'ko', ms=3)
+            for _1, _2, _3 in zip(atom_x, atom_y, atom_label):
+                ax2.text(_1, _2, _3)
+
+            plt.show()
+
+            return
+        qtb_1.clicked.connect(lambda: func_plot(obj))
+        w_actions.append(qtb_1)
+
     elif not(flag_crystals & flag_experiments):
         if not flag_crystals:
             qtb_1 = QtWidgets.QToolButton()
